import * as GameTest from "mojang-gametest";
import { BlockLocation, TicksPerSecond, Location } from "mojang-minecraft";
const WARDEN_TESTS_PADDING = 16; // The paddings is there to make sure vibrations don't interefere with the warden 


GameTest.register("WardenTests", "warden_despawn", (test) => {
    const wardenEntityType = "minecraft:warden";
    const startPos = new BlockLocation(3, 1, 3);
    test.spawn(wardenEntityType, startPos.above());

    test
        .startSequence()
        .thenWait(() => {
            test.assertEntityPresentInArea(wardenEntityType, false);
        })
        .thenSucceed();
}).maxTicks(TicksPerSecond * 100).tag(GameTest.Tags.suiteDefault).padding(WARDEN_TESTS_PADDING); //timeout after 100 seconds

GameTest.register("WardenTests", "warden_kill_moving_entity", (test) => {
    const wardenEntityType = "minecraft:warden";
    const pigEntityType = "minecraft:pig";
    const startPosWarden = new BlockLocation(1, 1, 1);
    const startPosPig = new Location(6, 2, 6);
    const walkPosPig = new Location(6, 2, 1);
    test.spawn(wardenEntityType, startPosWarden.above());
    const pig = test.spawnWithoutBehaviorsAtLocation(pigEntityType, startPosPig);

    let sequence = test.startSequence().thenIdle(1);

    for (let i = 1; i <= 10; i++) {
        sequence
            .thenExecute(() => {
                test.walkToLocation(pig, walkPosPig, 1);
            })
            .thenIdle(TicksPerSecond * 3)
            .thenExecute(() => {
                test.walkToLocation(pig, startPosPig, 1);
            })
            .thenIdle(TicksPerSecond * 3)
    }
    sequence
        .thenWait(() => {
            test.assertEntityPresentInArea(pigEntityType, false);
        })
        .thenSucceed();
}).maxTicks(TicksPerSecond * 90).tag(GameTest.Tags.suiteDefault).padding(WARDEN_TESTS_PADDING); //timeout after 90 seconds

GameTest.register("WardenTests", "warden_sniff_and_kill_static_entity", (test) => {
    const wardenEntityType = "minecraft:warden";
    const pigEntityType = "minecraft:pig";
    const startPosWarden = new BlockLocation(1, 1, 1);
    const startPosPig = new Location(7, 2, 7);
    test.spawn(wardenEntityType, startPosWarden.above());
    test.spawnWithoutBehaviorsAtLocation(pigEntityType, startPosPig);

    test
        .startSequence()
        .thenWait(() => {
            test.assertEntityPresentInArea(pigEntityType, false);
        })
        .thenSucceed();
}).maxTicks(TicksPerSecond * 60).tag(GameTest.Tags.suiteDefault).padding(WARDEN_TESTS_PADDING); //timeout after 60 seconds

GameTest.register("WardenTests", "warden_sniff_and_kill_player_before_mob", (test) => {
    const wardenEntityType = "minecraft:warden";
    const pigEntityType = "minecraft:pig";
    const startPosWarden = new BlockLocation(1, 1, 1);
    const startPosPlayer = new BlockLocation(1, 2, 6);
    const startPosPig = new Location(6, 2, 6);
    test.spawn(wardenEntityType, startPosWarden.above());
    test.spawnWithoutBehaviorsAtLocation(pigEntityType, startPosPig);
    test.spawnSimulatedPlayer(startPosPlayer, "playerSim_warden");

    test
        .startSequence()
        .thenWait(() => {
            test.assertEntityPresentInArea("minecraft:player", false);
        })
        .thenWait(() => {
            test.assertEntityPresentInArea("minecraft:pig", true);
        })
        .thenSucceed();
}).maxTicks(TicksPerSecond * 60).tag(GameTest.Tags.suiteDefault).padding(WARDEN_TESTS_PADDING); //timeout after 60 seconds

GameTest.register("WardenTests", "warden_go_to_projectile", (test) => {
    const wardenEntityType = "minecraft:warden";
    const startPosWarden = new BlockLocation(1, 1, 1);
    const snowballEntityType = "minecraft:snowball";
    // spawns snowball above the ground so that it falls down and breaks
    const startPosSnowball = new BlockLocation(7, 4, 7);
    test.spawn(wardenEntityType, startPosWarden.above());
    test.spawn(snowballEntityType, startPosSnowball);

    test
        .startSequence()
        .thenWait(() => {
            test.assertEntityPresent(wardenEntityType, startPosSnowball, true);
        })
        .thenSucceed();
}).maxTicks(TicksPerSecond * 10).padding(WARDEN_TESTS_PADDING); //timeout after 10 seconds

GameTest.register("WardenTests", "warden_path_lava", (test) => {
    const wardenEntityType = "minecraft:warden";
    const pigEntityType = "minecraft:pig";
    const startPosWarden = new BlockLocation(1, 3, 2);
    const startPosPig = new Location(7, 3, 2);
    test.spawn(wardenEntityType, startPosWarden.above());
    test.spawnWithoutBehaviorsAtLocation(pigEntityType, startPosPig);

    test
        .startSequence()
        .thenWait(() => {
            test.assertEntityPresentInArea("minecraft:pig", false);
        })
        .thenSucceed();
}).maxTicks(TicksPerSecond * 60).tag(GameTest.Tags.suiteDefault).padding(WARDEN_TESTS_PADDING); //timeout after 60 seconds

GameTest.register("WardenTests", "warden_path_water", (test) => {
    const wardenEntityType = "minecraft:warden";
    const pigEntityType = "minecraft:pig";
    const startPosWarden = new BlockLocation(1, 3, 2);
    const startPosPig = new Location(7, 3, 2);
    test.spawn(wardenEntityType, startPosWarden.above());
    test.spawnWithoutBehaviorsAtLocation(pigEntityType, startPosPig);

    test
        .startSequence()
        .thenWait(() => {
            test.assertEntityPresentInArea("minecraft:pig", false);
        })
        .thenSucceed();
}).maxTicks(TicksPerSecond * 60).tag(GameTest.Tags.suiteDefault).padding(WARDEN_TESTS_PADDING); //timeout after 60 seconds

// SIG // Begin signature block
// SIG // MIInvQYJKoZIhvcNAQcCoIInrjCCJ6oCAQExDzANBglg
// SIG // hkgBZQMEAgEFADB3BgorBgEEAYI3AgEEoGkwZzAyBgor
// SIG // BgEEAYI3AgEeMCQCAQEEEBDgyQbOONQRoqMAEEvTUJAC
// SIG // AQACAQACAQACAQACAQAwMTANBglghkgBZQMEAgEFAAQg
// SIG // gE4yGadNcnUmSQejnSfDsLroiXuP7KMznQ6FzcDbiWmg
// SIG // gg2BMIIF/zCCA+egAwIBAgITMwAAAsyOtZamvdHJTgAA
// SIG // AAACzDANBgkqhkiG9w0BAQsFADB+MQswCQYDVQQGEwJV
// SIG // UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
// SIG // UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
// SIG // cmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBT
// SIG // aWduaW5nIFBDQSAyMDExMB4XDTIyMDUxMjIwNDYwMVoX
// SIG // DTIzMDUxMTIwNDYwMVowdDELMAkGA1UEBhMCVVMxEzAR
// SIG // BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
// SIG // bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
// SIG // bjEeMBwGA1UEAxMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
// SIG // ok2x7OvGwA7zbnfezc3HT9M4dJka+FaQ7+vCqG40Bcm1
// SIG // QLlYIiDX/Whts0LVijaOvtl9iMeuShnAV7mchItKAVAA
// SIG // BpyHuTuav2NCI9FsA8jFmlWndk3uK9RInNx1h1H4ojYx
// SIG // dBExyoN6muwwslKsLEfauUml7h5WAsDPpufTZd4yp2Jy
// SIG // iy384Zdd8CJlfQxfDe+gDZEciugWKHPSOoRxdjAk0GFm
// SIG // 0OH14MyoYM4+M3mm1oH7vmSQohS5KIL3NEVW9Mdw7csT
// SIG // G5f93uORLvrJ/8ehFcGyWVb7UGHJnRhdcgGIbfiZzZls
// SIG // AMS/DIBzM8RHKGNUNSbbLYmN/rt7pRjL4QIDAQABo4IB
// SIG // fjCCAXowHwYDVR0lBBgwFgYKKwYBBAGCN0wIAQYIKwYB
// SIG // BQUHAwMwHQYDVR0OBBYEFIi4R40ylsyKlSKfrDNqzhx9
// SIG // da30MFAGA1UdEQRJMEekRTBDMSkwJwYDVQQLEyBNaWNy
// SIG // b3NvZnQgT3BlcmF0aW9ucyBQdWVydG8gUmljbzEWMBQG
// SIG // A1UEBRMNMjMwMDEyKzQ3MDUyOTAfBgNVHSMEGDAWgBRI
// SIG // bmTlUAXTgqoXNzcitW2oynUClTBUBgNVHR8ETTBLMEmg
// SIG // R6BFhkNodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtp
// SIG // b3BzL2NybC9NaWNDb2RTaWdQQ0EyMDExXzIwMTEtMDct
// SIG // MDguY3JsMGEGCCsGAQUFBwEBBFUwUzBRBggrBgEFBQcw
// SIG // AoZFaHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraW9w
// SIG // cy9jZXJ0cy9NaWNDb2RTaWdQQ0EyMDExXzIwMTEtMDct
// SIG // MDguY3J0MAwGA1UdEwEB/wQCMAAwDQYJKoZIhvcNAQEL
// SIG // BQADggIBAHgPA7DgB0udzEyB2LvG216zuskLUQ+iX8jF
// SIG // nl2i7tzXPDw5xXNXn2KvxdzBsf2osDW3LCdjFOwSjVkz
// SIG // +SUFQQNhjSHkd5knF6pzrL9V6lz72XiEg1Vi2gUM3HiL
// SIG // XSMIKOgdd78ZZJEmDLwdA692MO/1vVOFpOSv0QzpyBr5
// SIG // iqiotwMMsZVdZqXn8u9vRSmlk+3nQXdyOPoZXTGPLHXw
// SIG // z41kbSc4zI12bONTlDsLR3HD2s44wuyp3c72R8f9FVi/
// SIG // J9DU/+NOL37Z1yonzGZEuKdrAd6CvupAnLMlrIEv93mB
// SIG // sNRXuDDp4p9UYYK1taxzzgyUxgFDpluMHN0Oiiq9s73u
// SIG // 7DA2XvbX8paJz8IZPe9a1/KhsOi5Kxhb99SCXiUnv2lG
// SIG // xnVAz5G6wAW1bzxJYKI+Xj90RKseY3X5EMO7TnVpIZ9I
// SIG // w1IdrkHp/QLY90ZCch7kdBlLCVTFhSXZCDv4BcM6DhpR
// SIG // zbJsb6QDVfOv9aoG9aGV3a1EacyaedzLA2gWP6cTnCdA
// SIG // r4OrlrN5EFoCpOWgc77F/eQc3SLR06VTLVT1uKuNVxL2
// SIG // xZlD9Z+qC+a3TXa0zI/x1zEZNSgpLGsdVcaN6r/td3Ar
// SIG // GQGkDWiAL7eS75LIWZA2SD//9B56uzZ1nmEd8+KBYsPT
// SIG // dp922/W2kFrlj7MBtA6vWE/ZG/grOKiCMIIHejCCBWKg
// SIG // AwIBAgIKYQ6Q0gAAAAAAAzANBgkqhkiG9w0BAQsFADCB
// SIG // iDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0
// SIG // b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1p
// SIG // Y3Jvc29mdCBDb3Jwb3JhdGlvbjEyMDAGA1UEAxMpTWlj
// SIG // cm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9yaXR5
// SIG // IDIwMTEwHhcNMTEwNzA4MjA1OTA5WhcNMjYwNzA4MjEw
// SIG // OTA5WjB+MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2Fz
// SIG // aGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UE
// SIG // ChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYDVQQD
// SIG // Ex9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBDQSAyMDEx
// SIG // MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEA
// SIG // q/D6chAcLq3YbqqCEE00uvK2WCGfQhsqa+laUKq4Bjga
// SIG // BEm6f8MMHt03a8YS2AvwOMKZBrDIOdUBFDFC04kNeWSH
// SIG // fpRgJGyvnkmc6Whe0t+bU7IKLMOv2akrrnoJr9eWWcpg
// SIG // GgXpZnboMlImEi/nqwhQz7NEt13YxC4Ddato88tt8zpc
// SIG // oRb0RrrgOGSsbmQ1eKagYw8t00CT+OPeBw3VXHmlSSnn
// SIG // Db6gE3e+lD3v++MrWhAfTVYoonpy4BI6t0le2O3tQ5GD
// SIG // 2Xuye4Yb2T6xjF3oiU+EGvKhL1nkkDstrjNYxbc+/jLT
// SIG // swM9sbKvkjh+0p2ALPVOVpEhNSXDOW5kf1O6nA+tGSOE
// SIG // y/S6A4aN91/w0FK/jJSHvMAhdCVfGCi2zCcoOCWYOUo2
// SIG // z3yxkq4cI6epZuxhH2rhKEmdX4jiJV3TIUs+UsS1Vz8k
// SIG // A/DRelsv1SPjcF0PUUZ3s/gA4bysAoJf28AVs70b1FVL
// SIG // 5zmhD+kjSbwYuER8ReTBw3J64HLnJN+/RpnF78IcV9uD
// SIG // jexNSTCnq47f7Fufr/zdsGbiwZeBe+3W7UvnSSmnEyim
// SIG // p31ngOaKYnhfsi+E11ecXL93KCjx7W3DKI8sj0A3T8Hh
// SIG // hUSJxAlMxdSlQy90lfdu+HggWCwTXWCVmj5PM4TasIgX
// SIG // 3p5O9JawvEagbJjS4NaIjAsCAwEAAaOCAe0wggHpMBAG
// SIG // CSsGAQQBgjcVAQQDAgEAMB0GA1UdDgQWBBRIbmTlUAXT
// SIG // gqoXNzcitW2oynUClTAZBgkrBgEEAYI3FAIEDB4KAFMA
// SIG // dQBiAEMAQTALBgNVHQ8EBAMCAYYwDwYDVR0TAQH/BAUw
// SIG // AwEB/zAfBgNVHSMEGDAWgBRyLToCMZBDuRQFTuHqp8cx
// SIG // 0SOJNDBaBgNVHR8EUzBRME+gTaBLhklodHRwOi8vY3Js
// SIG // Lm1pY3Jvc29mdC5jb20vcGtpL2NybC9wcm9kdWN0cy9N
// SIG // aWNSb29DZXJBdXQyMDExXzIwMTFfMDNfMjIuY3JsMF4G
// SIG // CCsGAQUFBwEBBFIwUDBOBggrBgEFBQcwAoZCaHR0cDov
// SIG // L3d3dy5taWNyb3NvZnQuY29tL3BraS9jZXJ0cy9NaWNS
// SIG // b29DZXJBdXQyMDExXzIwMTFfMDNfMjIuY3J0MIGfBgNV
// SIG // HSAEgZcwgZQwgZEGCSsGAQQBgjcuAzCBgzA/BggrBgEF
// SIG // BQcCARYzaHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3Br
// SIG // aW9wcy9kb2NzL3ByaW1hcnljcHMuaHRtMEAGCCsGAQUF
// SIG // BwICMDQeMiAdAEwAZQBnAGEAbABfAHAAbwBsAGkAYwB5
// SIG // AF8AcwB0AGEAdABlAG0AZQBuAHQALiAdMA0GCSqGSIb3
// SIG // DQEBCwUAA4ICAQBn8oalmOBUeRou09h0ZyKbC5YR4WOS
// SIG // mUKWfdJ5DJDBZV8uLD74w3LRbYP+vj/oCso7v0epo/Np
// SIG // 22O/IjWll11lhJB9i0ZQVdgMknzSGksc8zxCi1LQsP1r
// SIG // 4z4HLimb5j0bpdS1HXeUOeLpZMlEPXh6I/MTfaaQdION
// SIG // 9MsmAkYqwooQu6SpBQyb7Wj6aC6VoCo/KmtYSWMfCWlu
// SIG // WpiW5IP0wI/zRive/DvQvTXvbiWu5a8n7dDd8w6vmSiX
// SIG // mE0OPQvyCInWH8MyGOLwxS3OW560STkKxgrCxq2u5bLZ
// SIG // 2xWIUUVYODJxJxp/sfQn+N4sOiBpmLJZiWhub6e3dMNA
// SIG // BQamASooPoI/E01mC8CzTfXhj38cbxV9Rad25UAqZaPD
// SIG // XVJihsMdYzaXht/a8/jyFqGaJ+HNpZfQ7l1jQeNbB5yH
// SIG // PgZ3BtEGsXUfFL5hYbXw3MYbBL7fQccOKO7eZS/sl/ah
// SIG // XJbYANahRr1Z85elCUtIEJmAH9AAKcWxm6U/RXceNcbS
// SIG // oqKfenoi+kiVH6v7RyOA9Z74v2u3S5fi63V4GuzqN5l5
// SIG // GEv/1rMjaHXmr/r8i+sLgOppO6/8MO0ETI7f33VtY5E9
// SIG // 0Z1WTk+/gFcioXgRMiF670EKsT/7qMykXcGhiJtXcVZO
// SIG // SEXAQsmbdlsKgEhr/Xmfwb1tbWrJUnMTDXpQzTGCGZQw
// SIG // ghmQAgEBMIGVMH4xCzAJBgNVBAYTAlVTMRMwEQYDVQQI
// SIG // EwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4w
// SIG // HAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xKDAm
// SIG // BgNVBAMTH01pY3Jvc29mdCBDb2RlIFNpZ25pbmcgUENB
// SIG // IDIwMTECEzMAAALMjrWWpr3RyU4AAAAAAswwDQYJYIZI
// SIG // AWUDBAIBBQCggcAwGQYJKoZIhvcNAQkDMQwGCisGAQQB
// SIG // gjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcC
// SIG // ARUwLwYJKoZIhvcNAQkEMSIEIPi/2jxyTZakKBzUYpIh
// SIG // +ZRnu2hK9AzL7an2/IjrHyUfMFQGCisGAQQBgjcCAQwx
// SIG // RjBEoCSAIgBNAGkAbgBlAGMAcgBhAGYAdAAgAEIAZQBk
// SIG // AHIAbwBjAGuhHIAaaHR0cHM6Ly93d3cubWluZWNyYWZ0
// SIG // Lm5ldC8wDQYJKoZIhvcNAQEBBQAEggEAbwud4zOn7bQJ
// SIG // bUuueMfmIGoCY8GODHEHUEAXxkmH+LVxrIoqezxp53Zk
// SIG // zARNcRd49x3ZRbLu3Cu3NUgKDCYA/a1x5TnQjrJhtyLh
// SIG // MXKPo7CEXrm8NbXB3BwHnNNLiH0QuWPnfTwL8X65yoWh
// SIG // T/IQJt8IC1hV6ok1bAuns5fW9oS+6WKg1ECsBgW/AYVg
// SIG // go0wVEImK/JkCA+JSHUvMTqMmjPysSYCrzwrM/+kZySP
// SIG // XgQ52Eql75GDXD7SfMlb+SBl2NxcE3Mm39328a+lnmZp
// SIG // lXqe5NIyeFHROoSm1B3J7LWcC4G1vbcM5/OHPt7KfyUX
// SIG // PBJ2p0kEyS8Y/5+ZVNZcDKGCFwwwghcIBgorBgEEAYI3
// SIG // AwMBMYIW+DCCFvQGCSqGSIb3DQEHAqCCFuUwghbhAgED
// SIG // MQ8wDQYJYIZIAWUDBAIBBQAwggFVBgsqhkiG9w0BCRAB
// SIG // BKCCAUQEggFAMIIBPAIBAQYKKwYBBAGEWQoDATAxMA0G
// SIG // CWCGSAFlAwQCAQUABCDMGgEbLZRCNvJIANdkdMNM9g1I
// SIG // cjrlA++purBE8OycgQIGYv7ZonvRGBMyMDIyMDgyNTA0
// SIG // NTA1OS45NDFaMASAAgH0oIHUpIHRMIHOMQswCQYDVQQG
// SIG // EwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UE
// SIG // BxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENv
// SIG // cnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQgT3Bl
// SIG // cmF0aW9ucyBQdWVydG8gUmljbzEmMCQGA1UECxMdVGhh
// SIG // bGVzIFRTUyBFU046MzJCRC1FM0Q1LTNCMUQxJTAjBgNV
// SIG // BAMTHE1pY3Jvc29mdCBUaW1lLVN0YW1wIFNlcnZpY2Wg
// SIG // ghFfMIIHEDCCBPigAwIBAgITMwAAAa38301Y410y6QAB
// SIG // AAABrTANBgkqhkiG9w0BAQsFADB8MQswCQYDVQQGEwJV
// SIG // UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
// SIG // UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
// SIG // cmF0aW9uMSYwJAYDVQQDEx1NaWNyb3NvZnQgVGltZS1T
// SIG // dGFtcCBQQ0EgMjAxMDAeFw0yMjAzMDIxODUxMzZaFw0y
// SIG // MzA1MTExODUxMzZaMIHOMQswCQYDVQQGEwJVUzETMBEG
// SIG // A1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
// SIG // ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MSkwJwYDVQQLEyBNaWNyb3NvZnQgT3BlcmF0aW9ucyBQ
// SIG // dWVydG8gUmljbzEmMCQGA1UECxMdVGhhbGVzIFRTUyBF
// SIG // U046MzJCRC1FM0Q1LTNCMUQxJTAjBgNVBAMTHE1pY3Jv
// SIG // c29mdCBUaW1lLVN0YW1wIFNlcnZpY2UwggIiMA0GCSqG
// SIG // SIb3DQEBAQUAA4ICDwAwggIKAoICAQDonlMqpU0q1S4b
// SIG // 2O0zvL4Avk+Tf8vzF3kd6DcBNKyyNRIP4DOYPTYT/iqq
// SIG // iXP+7jy+E7KAC2/kVX+q6GDZZckK2JqrlI8LKtutjA+S
// SIG // /Zfa1NaZc9rTD3oT/GnXVZTSI2CAYdDQuAsjBOrcVaaU
// SIG // K+3hXf21tF/bZ2ctWg1vs6GiQdhWPIWqJKuXETjRFuLq
// SIG // FbE017CApG2DgGH3yCBmpDBV1bwd9DXLLog5+rg5PN11
// SIG // 07NLZ0q/Bccz6A/EoDzyHFmljxIwkC6+SNAcX3VJYsDT
// SIG // xyDxIJaqLKwpUY1xu63GVm1njg4AE4tIug1LL9Vp9Czl
// SIG // uLLyuP1rnP7/XZZriK2pr/cIW4AspFLmGcvDMU87EKlN
// SIG // kcXJvIfPQlwOY39ZO5N2Ymi1CdNNzI7U7TV85YG/pNQO
// SIG // Xi9extwuBI8OQKFYjPlARkL8bk8aRXmhqEBanjhhrRCt
// SIG // KpNMHVF/af+ALvV7JXuQ+X78qQXbHFwbJVAXE+vgmKtJ
// SIG // EPbJrE8oVeAyvcG/WZ+zmxDn09AEMqaSOo8MwYx4QBhc
// SIG // qymIyG4JTGhIsY0b+13TYnNlbmCsku9QNIbOnW2w8f1e
// SIG // 4Q23b9rK7WdlUztMthPuEKjESrQAb+AIII2Cs+U2BrkO
// SIG // FW2z2PHwTvV6r2txp8dWQzkN5xlRSh8Gqxg3DXQd6nrL
// SIG // 7SWyBiYCYwIDAQABo4IBNjCCATIwHQYDVR0OBBYEFH1Q
// SIG // dB0EvKohWm5qhKjlfS6yxsFZMB8GA1UdIwQYMBaAFJ+n
// SIG // FV0AXmJdg/Tl0mWnG1M1GelyMF8GA1UdHwRYMFYwVKBS
// SIG // oFCGTmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2lv
// SIG // cHMvY3JsL01pY3Jvc29mdCUyMFRpbWUtU3RhbXAlMjBQ
// SIG // Q0ElMjAyMDEwKDEpLmNybDBsBggrBgEFBQcBAQRgMF4w
// SIG // XAYIKwYBBQUHMAKGUGh0dHA6Ly93d3cubWljcm9zb2Z0
// SIG // LmNvbS9wa2lvcHMvY2VydHMvTWljcm9zb2Z0JTIwVGlt
// SIG // ZS1TdGFtcCUyMFBDQSUyMDIwMTAoMSkuY3J0MAwGA1Ud
// SIG // EwEB/wQCMAAwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDQYJ
// SIG // KoZIhvcNAQELBQADggIBAIvQHUW7Mf4DknQV53cEXo0L
// SIG // rrUKnHt5N24LNbJT42UsehQ16dSGc4zsk9SaP93lFgwR
// SIG // SXYvh3rtTaMg3Rp/by+q8ZctS+vCuDJ3ywZvsm8ozfsW
// SIG // tXjWuPuHDqDrsRNirfI7ZmWyIcdo72OxfamP5Wp0eT0m
// SIG // 1CsgpYTUIcbJzVKfyUkPqO5wLkCfsLAsDwq12BOwvk7y
// SIG // g42unullNCuEYgVxRlNc+jLdyvcJTA/0BlOPvyBmQ5hP
// SIG // v2f5NCXyY2csQgJXJoXt64HYQ8wLBsSWKuD25RmUwXa/
// SIG // MJEMKT9o4IMjDGsDDOQ4IML12g266gkPIJLDYQmc06n0
// SIG // tnW4CxdJux38JVDK3J84v23U0kVsyF4OULB9beV2miB/
// SIG // k2dbmQbcFyfBDl6+kvtgOqvEUFWNdCpwR+mKRhPToVz3
// SIG // 7iLvPNprhRHYeBF2z3QEVEdSDEiMYPPJIUgQv2AcEIQq
// SIG // Y05LKuSd2fo8h1meDJ11+UeNpgEefOIkWHO2oGL7qLQx
// SIG // KnA9FPZSI4Ft9T6n6mUNbbmGU4hnChPUHGBr2RSp1XZK
// SIG // ivFdDPXvpBeQqgzqzDGEVdviPxYckLCR8Svxudw9DuW7
// SIG // 3SIdoA5xB0QBYoXWY9vWe1tKWhj6LRl9KAea/cKjYzN6
// SIG // 277hHuhSd9tzD2NcffzdYNduk/0qJKgAyQUyqsgWDeMz
// SIG // MIIHcTCCBVmgAwIBAgITMwAAABXF52ueAptJmQAAAAAA
// SIG // FTANBgkqhkiG9w0BAQsFADCBiDELMAkGA1UEBhMCVVMx
// SIG // EzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1Jl
// SIG // ZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3Jh
// SIG // dGlvbjEyMDAGA1UEAxMpTWljcm9zb2Z0IFJvb3QgQ2Vy
// SIG // dGlmaWNhdGUgQXV0aG9yaXR5IDIwMTAwHhcNMjEwOTMw
// SIG // MTgyMjI1WhcNMzAwOTMwMTgzMjI1WjB8MQswCQYDVQQG
// SIG // EwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UE
// SIG // BxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENv
// SIG // cnBvcmF0aW9uMSYwJAYDVQQDEx1NaWNyb3NvZnQgVGlt
// SIG // ZS1TdGFtcCBQQ0EgMjAxMDCCAiIwDQYJKoZIhvcNAQEB
// SIG // BQADggIPADCCAgoCggIBAOThpkzntHIhC3miy9ckeb0O
// SIG // 1YLT/e6cBwfSqWxOdcjKNVf2AX9sSuDivbk+F2Az/1xP
// SIG // x2b3lVNxWuJ+Slr+uDZnhUYjDLWNE893MsAQGOhgfWpS
// SIG // g0S3po5GawcU88V29YZQ3MFEyHFcUTE3oAo4bo3t1w/Y
// SIG // JlN8OWECesSq/XJprx2rrPY2vjUmZNqYO7oaezOtgFt+
// SIG // jBAcnVL+tuhiJdxqD89d9P6OU8/W7IVWTe/dvI2k45GP
// SIG // sjksUZzpcGkNyjYtcI4xyDUoveO0hyTD4MmPfrVUj9z6
// SIG // BVWYbWg7mka97aSueik3rMvrg0XnRm7KMtXAhjBcTyzi
// SIG // YrLNueKNiOSWrAFKu75xqRdbZ2De+JKRHh09/SDPc31B
// SIG // mkZ1zcRfNN0Sidb9pSB9fvzZnkXftnIv231fgLrbqn42
// SIG // 7DZM9ituqBJR6L8FA6PRc6ZNN3SUHDSCD/AQ8rdHGO2n
// SIG // 6Jl8P0zbr17C89XYcz1DTsEzOUyOArxCaC4Q6oRRRuLR
// SIG // vWoYWmEBc8pnol7XKHYC4jMYctenIPDC+hIK12NvDMk2
// SIG // ZItboKaDIV1fMHSRlJTYuVD5C4lh8zYGNRiER9vcG9H9
// SIG // stQcxWv2XFJRXRLbJbqvUAV6bMURHXLvjflSxIUXk8A8
// SIG // FdsaN8cIFRg/eKtFtvUeh17aj54WcmnGrnu3tz5q4i6t
// SIG // AgMBAAGjggHdMIIB2TASBgkrBgEEAYI3FQEEBQIDAQAB
// SIG // MCMGCSsGAQQBgjcVAgQWBBQqp1L+ZMSavoKRPEY1Kc8Q
// SIG // /y8E7jAdBgNVHQ4EFgQUn6cVXQBeYl2D9OXSZacbUzUZ
// SIG // 6XIwXAYDVR0gBFUwUzBRBgwrBgEEAYI3TIN9AQEwQTA/
// SIG // BggrBgEFBQcCARYzaHR0cDovL3d3dy5taWNyb3NvZnQu
// SIG // Y29tL3BraW9wcy9Eb2NzL1JlcG9zaXRvcnkuaHRtMBMG
// SIG // A1UdJQQMMAoGCCsGAQUFBwMIMBkGCSsGAQQBgjcUAgQM
// SIG // HgoAUwB1AGIAQwBBMAsGA1UdDwQEAwIBhjAPBgNVHRMB
// SIG // Af8EBTADAQH/MB8GA1UdIwQYMBaAFNX2VsuP6KJcYmjR
// SIG // PZSQW9fOmhjEMFYGA1UdHwRPME0wS6BJoEeGRWh0dHA6
// SIG // Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3JsL3Byb2R1
// SIG // Y3RzL01pY1Jvb0NlckF1dF8yMDEwLTA2LTIzLmNybDBa
// SIG // BggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKGPmh0dHA6
// SIG // Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2VydHMvTWlj
// SIG // Um9vQ2VyQXV0XzIwMTAtMDYtMjMuY3J0MA0GCSqGSIb3
// SIG // DQEBCwUAA4ICAQCdVX38Kq3hLB9nATEkW+Geckv8qW/q
// SIG // XBS2Pk5HZHixBpOXPTEztTnXwnE2P9pkbHzQdTltuw8x
// SIG // 5MKP+2zRoZQYIu7pZmc6U03dmLq2HnjYNi6cqYJWAAOw
// SIG // Bb6J6Gngugnue99qb74py27YP0h1AdkY3m2CDPVtI1Tk
// SIG // eFN1JFe53Z/zjj3G82jfZfakVqr3lbYoVSfQJL1AoL8Z
// SIG // thISEV09J+BAljis9/kpicO8F7BUhUKz/AyeixmJ5/AL
// SIG // aoHCgRlCGVJ1ijbCHcNhcy4sa3tuPywJeBTpkbKpW99J
// SIG // o3QMvOyRgNI95ko+ZjtPu4b6MhrZlvSP9pEB9s7GdP32
// SIG // THJvEKt1MMU0sHrYUP4KWN1APMdUbZ1jdEgssU5HLcEU
// SIG // BHG/ZPkkvnNtyo4JvbMBV0lUZNlz138eW0QBjloZkWsN
// SIG // n6Qo3GcZKCS6OEuabvshVGtqRRFHqfG3rsjoiV5PndLQ
// SIG // THa1V1QJsWkBRH58oWFsc/4Ku+xBZj1p/cvBQUl+fpO+
// SIG // y/g75LcVv7TOPqUxUYS8vwLBgqJ7Fx0ViY1w/ue10Cga
// SIG // iQuPNtq6TPmb/wrpNPgkNWcr4A245oyZ1uEi6vAnQj0l
// SIG // lOZ0dFtq0Z4+7X6gMTN9vMvpe784cETRkPHIqzqKOghi
// SIG // f9lwY1NNje6CbaUFEMFxBmoQtB1VM1izoXBm8qGCAtIw
// SIG // ggI7AgEBMIH8oYHUpIHRMIHOMQswCQYDVQQGEwJVUzET
// SIG // MBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVk
// SIG // bW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0
// SIG // aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQgT3BlcmF0aW9u
// SIG // cyBQdWVydG8gUmljbzEmMCQGA1UECxMdVGhhbGVzIFRT
// SIG // UyBFU046MzJCRC1FM0Q1LTNCMUQxJTAjBgNVBAMTHE1p
// SIG // Y3Jvc29mdCBUaW1lLVN0YW1wIFNlcnZpY2WiIwoBATAH
// SIG // BgUrDgMCGgMVAECS0a1FUeGWwQ4Fj47jalXLVM6QoIGD
// SIG // MIGApH4wfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldh
// SIG // c2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNV
// SIG // BAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UE
// SIG // AxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAw
// SIG // DQYJKoZIhvcNAQEFBQACBQDmsUDlMCIYDzIwMjIwODI1
// SIG // MDQyOTI1WhgPMjAyMjA4MjYwNDI5MjVaMHcwPQYKKwYB
// SIG // BAGEWQoEATEvMC0wCgIFAOaxQOUCAQAwCgIBAAICI44C
// SIG // Af8wBwIBAAICEkQwCgIFAOaykmUCAQAwNgYKKwYBBAGE
// SIG // WQoEAjEoMCYwDAYKKwYBBAGEWQoDAqAKMAgCAQACAweh
// SIG // IKEKMAgCAQACAwGGoDANBgkqhkiG9w0BAQUFAAOBgQAV
// SIG // n1wO1SuTXpTdLZMTMwF4mVj/DOX4zZFn8wP6br7xieW0
// SIG // 96o6vV+LWVKLqtAjMDy5dphCVqi/qCRd7zPyxg5DnV20
// SIG // xtW4G6yHXTOwa7jJw3WTffaduSf6b4UwdK8W7n2tjpCu
// SIG // iNipws2WDoMow19PICHVCgHy2k3bKGGLPzBcBzGCBA0w
// SIG // ggQJAgEBMIGTMHwxCzAJBgNVBAYTAlVTMRMwEQYDVQQI
// SIG // EwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4w
// SIG // HAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xJjAk
// SIG // BgNVBAMTHU1pY3Jvc29mdCBUaW1lLVN0YW1wIFBDQSAy
// SIG // MDEwAhMzAAABrfzfTVjjXTLpAAEAAAGtMA0GCWCGSAFl
// SIG // AwQCAQUAoIIBSjAaBgkqhkiG9w0BCQMxDQYLKoZIhvcN
// SIG // AQkQAQQwLwYJKoZIhvcNAQkEMSIEIOGFI3DLW/g2nntQ
// SIG // 2b5FjCHLzGqsURifYHrsBy8vFUw5MIH6BgsqhkiG9w0B
// SIG // CRACLzGB6jCB5zCB5DCBvQQgn+p8PQkeXtfjrbXJ9cPf
// SIG // dipp/GDE2Pw1jyB3jGzLUIIwgZgwgYCkfjB8MQswCQYD
// SIG // VQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4G
// SIG // A1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0
// SIG // IENvcnBvcmF0aW9uMSYwJAYDVQQDEx1NaWNyb3NvZnQg
// SIG // VGltZS1TdGFtcCBQQ0EgMjAxMAITMwAAAa38301Y410y
// SIG // 6QABAAABrTAiBCBLjysV3qwMFXc7S+g0bS1WJ4dKGRmv
// SIG // 1pKUNr+XYTVlrTANBgkqhkiG9w0BAQsFAASCAgBMpTmZ
// SIG // EjW6Xwy7NeSTwcG1LbbY6AZ9cQ9ytwlDAjcBbn9WUGoV
// SIG // m0iI+1iR7pDHS417sFdyjRG1K41IfKRv0ntizMcmigJ4
// SIG // t2Eu8V5tJj2g5HNtGRySgaipkOsRsFLPxM72aRj5bi7K
// SIG // 6zmkC6ooNXbFXOYZxq+MOdbJzpjW9HV9tDqFFmndJ0ZS
// SIG // gmJBhV0PBEjjioYxNTGIP4Riz/9CA/p02SDqGCva6Ti5
// SIG // LBcqLop/8cTfGH7xrT/V80x8N0gMdj3L0ug5FD/mPweP
// SIG // XacOk4QOIuSPm+M+MEneCvUGJ0zI1G0Zw+P5NbpVJrr3
// SIG // /CB01Qv84Q5OwRSnU0k2A/8qpS2bxcckIL0pLTe8eJIR
// SIG // 50T9UegSBFbNGINGQuCP/wqSeXs7dPuTSBMoBHa477No
// SIG // EZkzpUrHU8jZ1DO7RBXIxlBXVTvQ6D0BfwIzjGB52iFR
// SIG // f4FD+lFBAVwLueq8d5jg1tTfxv/aTqmwqTXRNgfygDub
// SIG // ZdJ5ms3RkBIpRO3F9XndJ0BO0hJSxYX3Tt6vBNZXB302
// SIG // FX+b8pomtROMzNotVnyPZap6Era/F9DMX3NeCNJzh5+y
// SIG // 0eCYQW483U7KTZRadJ+C+pebCzPzG4M5MJeI26R6QyWm
// SIG // E4fCr+r0R/jwUWc0sJjgv3MZ7OjpHihzmlJO6+vrhFVG
// SIG // hb8WRDFE2EY4l9/VEg==
// SIG // End signature block
